@@ -3,16 +3,27 @@
 Note: this module is named 'osm' instead of 'os' to avoid a collision with the
 builtin.
 """
-# Copyright (c) IPython Development Team.
-# Distributed under the terms of the Modified BSD License.
+from __future__ import print_function
+#-----------------------------------------------------------------------------
+#  Copyright (c) 2012 The IPython Development Team.
+#
+#  Distributed under the terms of the Modified BSD License.
+#
+#  The full license is in the file COPYING.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Stdlib
 import io
 import os
-import pathlib
 import re
 import sys
 from pprint import pformat
 
+# Our own packages
 from IPython.core import magic_arguments
 from IPython.core import oinspect
 from IPython.core import page
@@ -24,73 +35,18 @@ from IPython.core.magic import  (
 from IPython.testing.skipdoctest import skip_doctest
 from IPython.utils.openpy import source_to_unicode
 from IPython.utils.process import abbrev_cwd
+from IPython.utils import py3compat
+from IPython.utils.py3compat import unicode_type
 from IPython.utils.terminal import set_term_title
-from traitlets import Bool
-from warnings import warn
-
 
+#-----------------------------------------------------------------------------
+# Magic implementation classes
+#-----------------------------------------------------------------------------
 @magics_class
 class OSMagics(Magics):
     """Magics to interact with the underlying OS (shell-type functionality).
     """
 
-    cd_force_quiet = Bool(False,
-        help="Force %cd magic to be quiet even if -q is not passed."
-    ).tag(config=True)
-
-    def __init__(self, shell=None, **kwargs):
-
-        # Now define isexec in a cross platform manner.
-        self.is_posix = False
-        self.execre = None
-        if os.name == 'posix':
-            self.is_posix = True
-        else:
-            try:
-                winext = os.environ['pathext'].replace(';','|').replace('.','')
-            except KeyError:
-                winext = 'exe|com|bat|py'
-            try:
-                self.execre = re.compile(r'(.*)\.(%s)$' % winext,re.IGNORECASE)
-            except re.error:
-                warn("Seems like your pathext environmental "
-                     "variable is malformed. Please check it to "
-                     "enable a proper handle of file extensions "
-                     "managed for your system")
-                winext = 'exe|com|bat|py'
-                self.execre = re.compile(r'(.*)\.(%s)$' % winext,re.IGNORECASE)
-
-        # call up the chain
-        super().__init__(shell=shell, **kwargs)
-
-
-    def _isexec_POSIX(self, file):
-        """
-        Test for executable on a POSIX system
-        """
-        if os.access(file.path, os.X_OK):
-            # will fail on maxOS if access is not X_OK
-            return file.is_file()
-        return False
-
-
-    
-    def _isexec_WIN(self, file):
-        """
-        Test for executable file on non POSIX system
-        """
-        return file.is_file() and self.execre.match(file.name) is not None
-
-    def isexec(self, file):
-        """
-        Test for executable file on non POSIX system
-        """
-        if self.is_posix:
-            return self._isexec_POSIX(file)
-        else:
-            return self._isexec_WIN(file)
-
-
     @skip_doctest
     @line_magic
     def alias(self, parameter_s=''):
@@ -128,7 +84,7 @@ class OSMagics(Magics):
         Aliases expand Python variables just like system calls using ! or !!
         do: all expressions prefixed with '$' get expanded.  For details of
         the semantic rules, see PEP-215:
-        https://peps.python.org/pep-0215/.  This is the library used by
+        http://www.python.org/peps/pep-0215.html.  This is the library used by
         IPython for variable expansion.  If you want to access a true shell
         variable, an extra $ is necessary to prevent its expansion by
         IPython::
@@ -140,21 +96,11 @@ class OSMagics(Magics):
           In [9]: show $$PATH
           /usr/local/lf9560/bin:/usr/local/intel/compiler70/ia32/bin:...
 
-        You can use the alias facility to access all of $PATH.  See the %rehashx
+        You can use the alias facility to acess all of $PATH.  See the %rehashx
         function, which automatically creates aliases for the contents of your
         $PATH.
 
-        If called with no parameters, %alias prints the current alias table
-        for your system.  For posix systems, the default aliases are 'cat',
-        'cp', 'mv', 'rm', 'rmdir', and 'mkdir', and other platform-specific
-        aliases are added.  For windows-based systems, the default aliases are
-        'copy', 'ddir', 'echo', 'ls', 'ldir', 'mkdir', 'ren', and 'rmdir'.
-
-        You can see the definition of alias by adding a question mark in the
-        end::
-
-          In [1]: cat?
-          Repr: <alias cat for 'cat'>"""
+        If called with no parameters, %alias prints the current alias table."""
 
         par = parameter_s.strip()
         if not par:
@@ -220,59 +166,65 @@ class OSMagics(Magics):
             os.environ.get('PATH','').split(os.pathsep)]
 
         syscmdlist = []
-        savedir = os.getcwd()
+        # Now define isexec in a cross platform manner.
+        if os.name == 'posix':
+            isexec = lambda fname:os.path.isfile(fname) and \
+                     os.access(fname,os.X_OK)
+        else:
+            try:
+                winext = os.environ['pathext'].replace(';','|').replace('.','')
+            except KeyError:
+                winext = 'exe|com|bat|py'
+            if 'py' not in winext:
+                winext += '|py'
+            execre = re.compile(r'(.*)\.(%s)$' % winext,re.IGNORECASE)
+            isexec = lambda fname:os.path.isfile(fname) and execre.match(fname)
+        savedir = py3compat.getcwd()
 
         # Now walk the paths looking for executables to alias.
         try:
             # write the whole loop for posix/Windows so we don't have an if in
             # the innermost part
-            if self.is_posix:
+            if os.name == 'posix':
                 for pdir in path:
                     try:
                         os.chdir(pdir)
+                        dirlist = os.listdir(pdir)
                     except OSError:
                         continue
-
-                    # for python 3.6+ rewrite to: with os.scandir(pdir) as dirlist:
-                    dirlist = os.scandir(path=pdir)
                     for ff in dirlist:
-                        if self.isexec(ff):
-                            fname = ff.name
+                        if isexec(ff):
                             try:
                                 # Removes dots from the name since ipython
                                 # will assume names with dots to be python.
-                                if not self.shell.alias_manager.is_alias(fname):
+                                if not self.shell.alias_manager.is_alias(ff):
                                     self.shell.alias_manager.define_alias(
-                                        fname.replace('.',''), fname)
+                                        ff.replace('.',''), ff)
                             except InvalidAliasError:
                                 pass
                             else:
-                                syscmdlist.append(fname)
+                                syscmdlist.append(ff)
             else:
                 no_alias = Alias.blacklist
                 for pdir in path:
                     try:
                         os.chdir(pdir)
+                        dirlist = os.listdir(pdir)
                     except OSError:
                         continue
-
-                    # for python 3.6+ rewrite to: with os.scandir(pdir) as dirlist:
-                    dirlist = os.scandir(pdir)
                     for ff in dirlist:
-                        fname = ff.name
-                        base, ext = os.path.splitext(fname)
-                        if self.isexec(ff) and base.lower() not in no_alias:
+                        base, ext = os.path.splitext(ff)
+                        if isexec(ff) and base.lower() not in no_alias:
                             if ext.lower() == '.exe':
-                                fname = base
+                                ff = base
                                 try:
                                     # Removes dots from the name since ipython
                                     # will assume names with dots to be python.
                                     self.shell.alias_manager.define_alias(
-                                        base.lower().replace('.',''), fname)
+                                        base.lower().replace('.',''), ff)
                                 except InvalidAliasError:
                                     pass
-                                syscmdlist.append(fname)
-
+                                syscmdlist.append(ff)
             self.shell.db['syscmdlist'] = syscmdlist
         finally:
             os.chdir(savedir)
@@ -289,10 +241,7 @@ class OSMagics(Magics):
           In [9]: pwd
           Out[9]: '/home/tsuser/sprint/ipython'
         """
-        try:
-            return os.getcwd()
-        except FileNotFoundError as e:
-            raise UsageError("CWD no longer exists - please use %cd to change directory.") from e
+        return py3compat.getcwd()
 
     @skip_doctest
     @line_magic
@@ -300,34 +249,33 @@ class OSMagics(Magics):
         """Change the current working directory.
 
         This command automatically maintains an internal list of directories
-        you visit during your IPython session, in the variable ``_dh``. The
-        command :magic:`%dhist` shows this history nicely formatted. You can
-        also do ``cd -<tab>`` to see directory history conveniently.
+        you visit during your IPython session, in the variable _dh. The
+        command %dhist shows this history nicely formatted. You can also
+        do 'cd -<tab>' to see directory history conveniently.
+
         Usage:
 
-          - ``cd 'dir'``: changes to directory 'dir'.
-          - ``cd -``: changes to the last visited directory.
-          - ``cd -<n>``: changes to the n-th directory in the directory history.
-          - ``cd --foo``: change to directory that matches 'foo' in history
-          - ``cd -b <bookmark_name>``: jump to a bookmark set by %bookmark
-          - Hitting a tab key after ``cd -b`` allows you to tab-complete
-            bookmark names.
+          cd 'dir': changes to directory 'dir'.
 
-          .. note::
-            ``cd <bookmark_name>`` is enough if there is no directory
-            ``<bookmark_name>``, but a bookmark with the name exists.
+          cd -: changes to the last visited directory.
+
+          cd -<n>: changes to the n-th directory in the directory history.
+
+          cd --foo: change to directory that matches 'foo' in history
+
+          cd -b <bookmark_name>: jump to a bookmark set by %bookmark
+             (note: cd <bookmark_name> is enough if there is no
+              directory <bookmark_name>, but a bookmark with the name exists.)
+              'cd -b <tab>' allows you to tab-complete bookmark names.
 
         Options:
 
-        -q               Be quiet. Do not print the working directory after the
-                         cd command is executed. By default IPython's cd
-                         command does print this directory, since the default
-                         prompts do not display path information.
+        -q: quiet.  Do not print the working directory after the cd command is
+        executed.  By default IPython's cd command does print this directory,
+        since the default prompts do not display path information.
 
-        .. note::
-           Note that ``!cd`` doesn't work for this purpose because the shell
-           where ``!command`` runs is immediately discarded after executing
-           'command'.
+        Note that !cd doesn't work for this purpose because the shell where
+        !command runs is immediately discarded after executing 'command'.
 
         Examples
         --------
@@ -337,12 +285,7 @@ class OSMagics(Magics):
           /home/tsuser/parent/child
         """
 
-        try:
-            oldcwd = os.getcwd()
-        except FileNotFoundError:
-            # Happens if the CWD has been deleted.
-            oldcwd = None
-
+        oldcwd = py3compat.getcwd()
         numcd = re.match(r'(-)(\d+)$',parameter_s)
         # jump in directory history by number
         if numcd:
@@ -385,8 +328,8 @@ class OSMagics(Magics):
         if ps == '-':
             try:
                 ps = self.shell.user_ns['_dh'][-2]
-            except IndexError as e:
-                raise UsageError('%cd -: No previous directory to change to.') from e
+            except IndexError:
+                raise UsageError('%cd -: No previous directory to change to.')
         # jump to bookmark if needed
         else:
             if not os.path.isdir(ps) or 'b' in opts:
@@ -406,11 +349,11 @@ class OSMagics(Magics):
             try:
                 os.chdir(os.path.expanduser(ps))
                 if hasattr(self.shell, 'term_title') and self.shell.term_title:
-                    set_term_title(self.shell.term_title_format.format(cwd=abbrev_cwd()))
+                    set_term_title('IPython: ' + abbrev_cwd())
             except OSError:
                 print(sys.exc_info()[1])
             else:
-                cwd = pathlib.Path.cwd()
+                cwd = py3compat.getcwd()
                 dhist = self.shell.user_ns['_dh']
                 if oldcwd != cwd:
                     dhist.append(cwd)
@@ -419,14 +362,14 @@ class OSMagics(Magics):
         else:
             os.chdir(self.shell.home_dir)
             if hasattr(self.shell, 'term_title') and self.shell.term_title:
-                set_term_title(self.shell.term_title_format.format(cwd="~"))
-            cwd = pathlib.Path.cwd()
+                set_term_title('IPython: ' + '~')
+            cwd = py3compat.getcwd()
             dhist = self.shell.user_ns['_dh']
 
             if oldcwd != cwd:
                 dhist.append(cwd)
                 self.shell.db['dhist'] = compress_dhist(dhist)[-100:]
-        if not 'q' in opts and not self.cd_force_quiet and self.shell.user_ns['_dh']:
+        if not 'q' in opts and self.shell.user_ns['_dh']:
             print(self.shell.user_ns['_dh'][-1])
 
     @line_magic
@@ -435,11 +378,11 @@ class OSMagics(Magics):
 
         Usage:\\
 
-          :``%env``: lists all environment variables/values
-          :``%env var``: get value for var
-          :``%env var val``: set value for var
-          :``%env var=val``: set value for var
-          :``%env var=$val``: set value for var, using python expansion if possible
+          %env: lists all environment variables/values
+          %env var: get value for var
+          %env var val: set value for var
+          %env var=val: set value for var
+          %env var=$val: set value for var, using python expansion if possible
         """
         if parameter_s.strip():
             split = '=' if '=' in parameter_s else ' '
@@ -453,13 +396,7 @@ class OSMagics(Magics):
                     raise UsageError(err)
             if len(bits) > 1:
                 return self.set_env(parameter_s)
-        env = dict(os.environ)
-        # hide likely secrets when printing the whole environment
-        for key in list(env):
-            if any(s in key.lower() for s in ('key', 'token', 'secret')):
-                env[key] = '<hidden>'
-
-        return env
+        return dict(os.environ)
 
     @line_magic
     def set_env(self, parameter_s):
@@ -468,9 +405,9 @@ class OSMagics(Magics):
         string.
 
         Usage:\\
-          :``%set_env var val``: set value for var
-          :``%set_env var=val``: set value for var
-          :``%set_env var=$val``: set value for var, using python expansion if possible
+          %set_env var val: set value for var
+          %set_env var=val: set value for var
+          %set_env var=$val: set value for var, using python expansion if possible
         """
         split = '=' if '=' in parameter_s else ' '
         bits = parameter_s.split(split, 1)
@@ -488,7 +425,7 @@ class OSMagics(Magics):
             err = "refusing to set env var with whitespace: '{0}'"
             err = err.format(val)
             raise UsageError(err)
-        os.environ[var] = val
+        os.environ[py3compat.cast_bytes_py2(var)] = py3compat.cast_bytes_py2(val)
         print('env: {0}={1}'.format(var,val))
 
     @line_magic
@@ -501,11 +438,11 @@ class OSMagics(Magics):
 
         dir_s = self.shell.dir_stack
         tgt = os.path.expanduser(parameter_s)
-        cwd = os.getcwd().replace(self.shell.home_dir,'~')
+        cwd = py3compat.getcwd().replace(self.shell.home_dir,'~')
         if tgt:
             self.cd(parameter_s)
         dir_s.insert(0,cwd)
-        return self.shell.run_line_magic('dirs', '')
+        return self.shell.magic('dirs')
 
     @line_magic
     def popd(self, parameter_s=''):
@@ -629,8 +566,8 @@ class OSMagics(Magics):
 
             # while the list form is useful to loop over:
             In [6]: for f in a.l:
-               ...:      !wc -l $f
-               ...:
+              ...:      !wc -l $f
+              ...:
             146 setup.py
             130 win32_manual_post_install.py
 
@@ -763,15 +700,15 @@ class OSMagics(Magics):
         if 'd' in opts:
             try:
                 todel = args[0]
-            except IndexError as e:
+            except IndexError:
                 raise UsageError(
-                    "%bookmark -d: must provide a bookmark to delete") from e
+                    "%bookmark -d: must provide a bookmark to delete")
             else:
                 try:
                     del bkms[todel]
-                except KeyError as e:
+                except KeyError:
                     raise UsageError(
-                        "%%bookmark -d: Can't delete bookmark '%s'" % todel) from e
+                        "%%bookmark -d: Can't delete bookmark '%s'" % todel)
 
         elif 'r' in opts:
             bkms = {}
@@ -789,7 +726,7 @@ class OSMagics(Magics):
             if not args:
                 raise UsageError("%bookmark: You must specify the bookmark name")
             elif len(args)==1:
-                bkms[args[0]] = os.getcwd()
+                bkms[args[0]] = py3compat.getcwd()
             elif len(args)==2:
                 bkms[args[0]] = args[1]
         self.shell.db['bookmarks'] = bkms
@@ -802,17 +739,18 @@ class OSMagics(Magics):
         to be Python source and will show it with syntax highlighting.
 
         This magic command can either take a local filename, an url,
-        an history range (see %history) or a macro as argument.
-
-        If no parameter is given, prints out history of current session up to
-        this point. ::
+        an history range (see %history) or a macro as argument ::
 
         %pycat myscript.py
         %pycat 7-27
         %pycat myMacro
         %pycat http://www.example.com/myscript.py
         """
-        try:
+        if not parameter_s:
+            raise UsageError('Missing filename, URL, input history range, '
+                             'or macro.')
+
+        try :
             cont = self.shell.find_user_code(parameter_s, skip_encoding_cookie=False)
         except (ValueError, IOError):
             print("Error: no such file, variable, URL, history range or macro")
@@ -827,21 +765,18 @@ class OSMagics(Magics):
              'The file will be created if it does not exist.'
     )
     @magic_arguments.argument(
-        'filename', type=str,
+        'filename', type=unicode_type,
         help='file to write'
     )
     @cell_magic
     def writefile(self, line, cell):
         """Write the contents of the cell to a file.
-
+        
         The file will be overwritten unless the -a (--append) flag is specified.
         """
         args = magic_arguments.parse_argstring(self.writefile, line)
-        if re.match(r'^(\'.*\')|(".*")$', args.filename):
-            filename = os.path.expanduser(args.filename[1:-1])
-        else:
-            filename = os.path.expanduser(args.filename)
-            
+        filename = os.path.expanduser(args.filename)
+
         if os.path.exists(filename):
             if args.append:
                 print("Appending to %s" % filename)
