@@ -1,6 +1,9 @@
+import nose.tools as nt
+
 from IPython.core.error import TryNext
 from IPython.lib.clipboard import ClipboardEmpty
 from IPython.testing.decorators import skip_if_no_x11
+from IPython.utils.py3compat import unicode_type
 
 @skip_if_no_x11
 def test_clipboard_get():
@@ -16,4 +19,4 @@ def test_clipboard_get():
         # No clipboard access API available
         pass
     else:
-        assert isinstance(a, str)
+        nt.assert_is_instance(a, unicode_type)
